@@ -1,5 +1,5 @@
 use futures_intrusive::sync::ManualResetEvent;
-use rnp::{RnpCore, RNP_NAME, RNP_AUTHOR, RNP_ABOUT};
+use rnp::{RnpCore, RNP_ABOUT, RNP_AUTHOR, RNP_NAME};
 use rnp_cli_options::RnpCliOptions;
 use std::sync::Arc;
 use structopt::StructOpt;
@@ -17,14 +17,15 @@ fn main() {
     let rnp_core_config = opts.to_rnp_core_config();
 
     let rt = Runtime::new().unwrap();
-    rt.block_on( async {
+    rt.block_on(async {
         let stop_event = Arc::new(ManualResetEvent::new(false));
         let mut rp = RnpCore::new(rnp_core_config, stop_event.clone());
 
         ctrlc::set_handler(move || {
             tracing::debug!("Ctrl+C received. Stopping all ping workers.");
             stop_event.set();
-        }).expect("Error setting Ctrl-C handler");
+        })
+        .expect("Error setting Ctrl-C handler");
 
         rp.run_warmup_pings().await;
 
