@@ -1,28 +1,46 @@
-use crate::{PingClient, PingClientConfig, RnpSupportedProtocol};
 use crate::ping_clients::ping_client_tcp::PingClientTcp;
+use crate::*;
 
 #[cfg(any(not(target_os = "windows"), not(target_arch = "aarch64")))]
 use crate::ping_clients::ping_client_quic::PingClientQuic;
 
-#[cfg(any(not(target_os = "windows"), not(target_arch = "aarch64")))]
 pub fn new(
-    protocol: RnpSupportedProtocol,
+    protocol: &RnpSupportedProtocol,
+    config: &PingClientConfig,
+    external_ping_client_factory: Option<ExternalPingClientFactory>,
+) -> Box<dyn PingClient + Send + Sync> {
+    if let Some(factory) = external_ping_client_factory {
+        if let Some(ping_client) = factory(protocol, config) {
+            return ping_client;
+        }
+    }
+
+    return new_inbox_ping_client(protocol, config);
+}
+
+#[cfg(any(not(target_os = "windows"), not(target_arch = "aarch64")))]
+fn new_inbox_ping_client(
+    protocol: &RnpSupportedProtocol,
     config: &PingClientConfig,
 ) -> Box<dyn PingClient + Send + Sync> {
     match protocol {
         RnpSupportedProtocol::TCP => return Box::new(PingClientTcp::new(config)),
         RnpSupportedProtocol::QUIC => return Box::new(PingClientQuic::new(config)),
+        RnpSupportedProtocol::External(p) => panic!("Protocol {} is not supported!", p),
     }
 }
 
 #[cfg(all(target_os = "windows", target_arch = "aarch64"))]
-pub fn new(
-    protocol: RnpSupportedProtocol,
+fn new_inbox_ping_client(
+    protocol: &RnpSupportedProtocol,
     config: &PingClientConfig,
 ) -> Box<dyn PingClient + Send + Sync> {
     match protocol {
         RnpSupportedProtocol::TCP => return Box::new(PingClientTcp::new(config)),
-        RnpSupportedProtocol::QUIC => panic!("Sorry, QUIC ping is not supported yet for Windows ARM64."),
+        RnpSupportedProtocol::QUIC => {
+            panic!("Sorry, QUIC ping is not supported yet for Windows ARM64.")
+        }
+        RnpSupportedProtocol::External(p) => panic!(format!("Protocol {} is not supported!", p)),
     }
 }
 
@@ -44,7 +62,7 @@ mod tests {
             use_timer_rtt: false,
         };
 
-        let ping_client = new(RnpSupportedProtocol::TCP, &config);
+        let ping_client = new(&RnpSupportedProtocol::TCP, &config, None);
         assert_eq!("TCP", ping_client.protocol());
     }
 }
