@@ -1,7 +1,6 @@
-use crate::ping_clients::ping_client::{PingClient, PingClientError, PingClientPingResultDetails, PingClientResult, PingClientWarning};
-use crate::PingClientConfig;
+use crate::*;
 use async_trait::async_trait;
-use quinn::{ClientConfigBuilder, Endpoint, EndpointError, ConnectionError};
+use quinn::{ClientConfig, ClientConfigBuilder, ConnectionError, Endpoint, EndpointError};
 use rustls::ServerCertVerified;
 use std::net::SocketAddr;
 use std::sync::Arc;
@@ -29,13 +28,34 @@ impl PingClientQuic {
             .map_err(|e| PingClientError::PreparationFailed(Box::new(e)))?;
         let server_name = self.config.server_name.as_ref().map_or("", |s| &s);
         let use_timer_rtt = self.config.use_timer_rtt;
-        let ping_result =
-            PingClientQuic::connect_to_target(&endpoint, source, target, server_name, use_timer_rtt).await;
+        let ping_result = PingClientQuic::connect_to_target(
+            &endpoint,
+            source,
+            target,
+            server_name,
+            use_timer_rtt,
+        )
+        .await;
         endpoint.wait_idle().await;
         return ping_result;
     }
 
     fn create_local_endpoint(&self, source: &SocketAddr) -> Result<Endpoint, EndpointError> {
+        let client_config = self.create_client_config();
+
+        let mut endpoint_builder = Endpoint::builder();
+        endpoint_builder.default_client_config(client_config);
+
+        let socket = std::net::UdpSocket::bind(source).map_err(EndpointError::Socket)?;
+        if let Some(ttl) = self.config.time_to_live {
+            socket.set_ttl(ttl).map_err(EndpointError::Socket)?;
+        }
+
+        let (endpoint, _) = endpoint_builder.with_socket(socket)?;
+        return Ok(endpoint);
+    }
+
+    fn create_client_config(&self) -> ClientConfig {
         let mut client_config_builder = ClientConfigBuilder::default();
 
         // Setup ALPN protocol if specified.
@@ -50,23 +70,22 @@ impl PingClientQuic {
 
         let mut client_config = client_config_builder.build();
         {
-            let tls_cfg: &mut rustls::ClientConfig =
-                Arc::get_mut(&mut client_config.crypto).unwrap();
+            let tls_cfg: &mut rustls::ClientConfig = Arc::get_mut(&mut client_config.crypto)
+                .expect("Failed to get QUIC client crypto config, which should never happen.");
+
             tls_cfg
                 .dangerous()
                 .set_certificate_verifier(Arc::new(SkipCertificationVerification));
 
-            let transport_config = Arc::get_mut(&mut client_config.transport).unwrap();
+            let transport_config = Arc::get_mut(&mut client_config.transport)
+                .expect("Failed to get QUIC client transport config, which should never happen.");
+
             transport_config
                 .max_idle_timeout(Some(self.config.wait_timeout))
-                .unwrap();
+                .expect("Failed to set QUIC client max idle timeout, which should never happen.");
         }
 
-        let mut endpoint_builder = Endpoint::builder();
-        endpoint_builder.default_client_config(client_config);
-
-        let (endpoint, _) = endpoint_builder.bind(source)?;
-        return Ok(endpoint);
+        return client_config;
     }
 
     async fn connect_to_target(
@@ -90,17 +109,25 @@ impl PingClientQuic {
         let connection = match connecting_result {
             Ok(connection) => Ok(connection),
             Err(e) => match e {
-                ConnectionError::TimedOut => Err(PingClientError::PingFailed(Box::new(e))),
+                ConnectionError::TimedOut => {
+                    return Ok(PingClientPingResultDetails::new(None, rtt, true, None));
+                },
                 ConnectionError::LocallyClosed => Err(PingClientError::PingFailed(Box::new(e))),
-                _ => return Ok(PingClientPingResultDetails::new(
-                    None,
-                    rtt,
-                    false,
-                    Some(PingClientWarning::AppHandshakeFailed(Box::new(e))))),
-            }
+                _ => {
+                    return Ok(PingClientPingResultDetails::new(
+                        None,
+                        rtt,
+                        false,
+                        Some(PingClientWarning::AppHandshakeFailed(Box::new(e))),
+                    ));
+                }
+            },
         }?;
 
-        let local_ip = connection.connection.local_ip().map_or(None, |addr| Some(SocketAddr::new(addr, source.port())));
+        let local_ip = connection
+            .connection
+            .local_ip()
+            .map_or(None, |addr| Some(SocketAddr::new(addr, source.port())));
         if !use_timer_rtt {
             rtt = connection.connection.rtt();
         }
@@ -136,3 +163,42 @@ impl rustls::ServerCertVerifier for SkipCertificationVerification {
         Ok(ServerCertVerified::assertion())
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::ping_clients::ping_client_test_common::*;
+    use crate::{ping_clients::ping_client_factory, PingClientConfig, RnpSupportedProtocol, rnp_test_common};
+    use std::time::Duration;
+    use tokio::runtime::Runtime;
+
+    #[test]
+    fn ping_client_quic_should_work() {
+        rnp_test_common::initialize();
+
+        let rt = Runtime::new().unwrap();
+
+        rt.block_on(async {
+            let config = PingClientConfig {
+                wait_timeout: Duration::from_millis(300),
+                time_to_live: None,
+                check_disconnect: false,
+                server_name: Some("localhost".to_string()),
+                log_tls_key: false,
+                alpn_protocol: Some("hq-29".to_string()),
+                use_timer_rtt: false,
+            };
+            let mut ping_client = ping_client_factory::new(&RnpSupportedProtocol::QUIC, &config, None);
+
+            // When connecting to a non existing port, on windows, it will timeout, but on other *nix OS, it will reject the connection.
+            let expected_results = ExpectedPingClientTestResults {
+                timeout_min_time: Duration::from_millis(200),
+                ping_non_existing_host_result: ExpectedTestCaseResult::Timeout,
+                ping_non_existing_port_result: ExpectedTestCaseResult::Timeout,
+                binding_invalid_source_ip_result: ExpectedTestCaseResult::Failed("failed to set up UDP socket: The requested address is not valid in its context. (os error 10049)"),
+                binding_unavailable_source_port_result: ExpectedTestCaseResult::Failed("failed to set up UDP socket: Only one usage of each socket address (protocol/network address/port) is normally permitted. (os error 10048)"),
+            };
+
+            run_ping_client_tests(&mut ping_client, &"127.0.0.1:4433".parse().unwrap(), &expected_results).await;
+        });
+    }
+}
