@@ -1,6 +1,4 @@
-use crate::ping_result_processors::ping_result_processor::PingResultProcessor;
-use crate::ping_clients::ping_client::PingClientWarning;
-use crate::PingResult;
+use crate::*;
 use std::io::{stdout, Write};
 use std::net::SocketAddr;
 use std::time::Instant;
@@ -116,6 +114,10 @@ impl PingResultProcessorConsoleLogger {
 }
 
 impl PingResultProcessor for PingResultProcessorConsoleLogger {
+    fn name(&self) -> &'static str {
+        "ConsoleLogger"
+    }
+
     fn process_ping_result(&mut self, ping_result: &PingResult) {
         self.update_statistics(ping_result);
         self.output_result_to_console(ping_result);
@@ -141,8 +143,18 @@ impl PingResultProcessor for PingResultProcessorConsoleLogger {
         let mut warning: String = String::from("");
         if self.handshake_failed_count > 0 || self.disconnect_failed_count > 0 {
             let mut warning_messages = Vec::new();
-            if self.handshake_failed_count > 0 { warning_messages.push(format!("App Handshake Failed = {}", self.handshake_failed_count)); }
-            if self.disconnect_failed_count > 0 { warning_messages.push(format!("Disconnect Failed = {}", self.disconnect_failed_count)); }
+            if self.handshake_failed_count > 0 {
+                warning_messages.push(format!(
+                    "App Handshake Failed = {}",
+                    self.handshake_failed_count
+                ));
+            }
+            if self.disconnect_failed_count > 0 {
+                warning_messages.push(format!(
+                    "Disconnect Failed = {}",
+                    self.disconnect_failed_count
+                ));
+            }
             warning = format!(" ({})", warning_messages.join(","));
         }
 
