@@ -1,7 +1,4 @@
-use crate::{
-    PingPortPicker, PingResult, PingResultProcessingWorker, PingResultProcessorConfig,
-    PingWorker, RnpCoreConfig,
-};
+use crate::*;
 use futures_intrusive::sync::ManualResetEvent;
 use std::sync::{Arc, Mutex};
 use tokio::{sync::mpsc, task::JoinHandle};
@@ -16,11 +13,79 @@ pub struct RnpCore {
 }
 
 impl RnpCore {
+    /// Take the config and stop event and generate the RnpCore which runs all the pings.
+    ///
+    /// # Arguments
+    ///
+    /// * `config`: The configuration of Rnp.
+    /// * `stop_event`: The event to signal Rnp to stop.
+    ///
+    /// returns: RnpCore
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use rnp::*;
+    /// use std::time::Duration;
+    /// use std::sync::Arc;
+    /// use futures_intrusive::sync::ManualResetEvent;
+    /// use tokio::runtime::Runtime;
+    ///
+    /// let config = RnpCoreConfig {
+    ///     worker_config: PingWorkerConfig {
+    ///         protocol: RnpSupportedProtocol::TCP,
+    ///         target: "10.0.0.1:443".parse().unwrap(),
+    ///         source_ip: "10.0.0.2".parse().unwrap(),
+    ///         ping_interval: Duration::from_millis(1500),
+    ///         ping_client_config: PingClientConfig {
+    ///             wait_timeout: Duration::from_millis(1000),
+    ///             time_to_live: Some(128),
+    ///             check_disconnect: false,
+    ///             server_name: None,
+    ///             log_tls_key: false,
+    ///             alpn_protocol: None,
+    ///             use_timer_rtt: false,
+    ///         },
+    ///     },
+    ///     worker_scheduler_config: PingWorkerSchedulerConfig {
+    ///         source_port_min: 1024,
+    ///         source_port_max: 2047,
+    ///         source_port_list: Some(vec![1024, 1025, 1026]),
+    ///         ping_count: Some(4),
+    ///         warmup_count: 1,
+    ///         parallel_ping_count: 1,
+    ///     },
+    ///     result_processor_config: PingResultProcessorConfig {
+    ///         no_console_log: false,
+    ///         csv_log_path: None,
+    ///         json_log_path: None,
+    ///         text_log_path: None,
+    ///         show_result_scatter: false,
+    ///         show_latency_scatter: false,
+    ///         latency_buckets: None,
+    ///     },
+    ///     external_ping_client_factory: None,
+    ///     extra_ping_result_processors: vec![],
+    /// };
+    ///
+    /// let rt = Runtime::new().unwrap();
+    /// rt.block_on(async {
+    ///     let stop_event = Arc::new(ManualResetEvent::new(false));
+    ///     let core = RnpCore::new(config, stop_event);
+    /// });
+    ///
+    /// ```
     #[tracing::instrument(name = "Start running Rnp core", level = "debug", skip(stop_event))]
-    pub fn new(config: RnpCoreConfig, stop_event: Arc<ManualResetEvent>) -> RnpCore {
+    pub fn new(mut config: RnpCoreConfig, stop_event: Arc<ManualResetEvent>) -> RnpCore {
+        // Move all extra ping result processors into another Vec for initializing result processing worker.
+        // Otherwise RnpCoreConfig will be partially moved and results in compile error.
+        let mut extra_ping_result_processors = Vec::new();
+        extra_ping_result_processors.append(&mut config.extra_ping_result_processors);
+
         let (result_sender, ping_result_processor_join_handle) =
             RnpCore::create_ping_result_processing_worker(
                 config.result_processor_config.clone(),
+                extra_ping_result_processors,
                 config.worker_scheduler_config.parallel_ping_count,
                 stop_event.clone(),
             );
@@ -38,9 +103,14 @@ impl RnpCore {
         return rnp_core;
     }
 
-    #[tracing::instrument(name = "Creating ping result processing worker", level = "debug")]
+    #[tracing::instrument(
+        name = "Creating ping result processing worker",
+        level = "debug",
+        skip(extra_ping_result_processors)
+    )]
     fn create_ping_result_processing_worker(
         result_processor_config: PingResultProcessorConfig,
+        extra_ping_result_processors: Vec<Box<dyn PingResultProcessor + Send + Sync>>,
         parallel_ping_count: u32,
         stop_event: Arc<ManualResetEvent>,
     ) -> (mpsc::Sender<PingResult>, JoinHandle<()>) {
@@ -53,6 +123,7 @@ impl RnpCore {
             mpsc::channel(ping_result_channel_size as usize);
         let ping_result_processor_join_handle = PingResultProcessingWorker::run(
             Arc::new(result_processor_config),
+            extra_ping_result_processors,
             stop_event,
             ping_result_receiver,
         );
@@ -61,13 +132,18 @@ impl RnpCore {
     }
 
     fn log_header_to_console(&self) {
+        let ttl_message = match self.config.worker_config.ping_client_config.time_to_live {
+            Some(ttl) => format!(" with TTL={}", ttl),
+            None => "".to_string(),
+        };
+
         println!(
-            "Start testing {} {:?}:",
-            self.config.worker_config.protocol,
-            self.config.worker_config.target
+            "Start testing {} {:?}{}:",
+            self.config.worker_config.protocol, self.config.worker_config.target, ttl_message,
         );
     }
 
+    /// Run all warm up pings one by one and wait until they are all completed.
     #[tracing::instrument(name = "Running warmup pings", level = "debug", skip(self))]
     pub async fn run_warmup_pings(&mut self) {
         let warmup_count = self.config.worker_scheduler_config.warmup_count;
@@ -99,11 +175,8 @@ impl RnpCore {
         tracing::debug!("Warmup ping completed!");
     }
 
-    #[tracing::instrument(
-        name = "Start running normal pings",
-        level = "debug",
-        skip(self)
-    )]
+    /// Start running all normal pings in the way that matches the specified config.
+    #[tracing::instrument(name = "Start running normal pings", level = "debug", skip(self))]
     pub fn start_running_normal_pings(&mut self) {
         if self.stop_event.is_set() {
             tracing::debug!("Stop event is signaled, skip running normal pings.");
@@ -145,6 +218,7 @@ impl RnpCore {
             let worker_join_handle = PingWorker::run(
                 worker_id,
                 shared_worker_config.clone(),
+                self.config.external_ping_client_factory.clone(),
                 source_port_picker.clone(),
                 self.stop_event.clone(),
                 self.result_sender.clone(),
@@ -156,6 +230,7 @@ impl RnpCore {
         return worker_join_handles;
     }
 
+    /// Wait for all pings to complete.
     #[tracing::instrument(
         name = "Waiting for RNP core to be stopped.",
         level = "debug",
