@@ -1,6 +1,6 @@
 use chrono::{DateTime, Utc};
+use serde::{Deserialize, Serialize};
 use std::net::IpAddr;
-use serde::{Serialize, Deserialize};
 
 #[derive(Debug, Serialize, Deserialize, PartialOrd, PartialEq)]
 #[serde(rename_all = "camelCase")]
@@ -40,4 +40,4 @@ pub struct PingResultCsvDto {
     pub ping_error: String,
     pub handshake_error: String,
     pub disconnect_error: String,
-}
\ No newline at end of file
+}
