@@ -1,9 +1,17 @@
-use crate::ping_clients::ping_client::PingClientError::{PingFailed, PreparationFailed};
-use crate::ping_result::PingResult;
+use crate::*;
 use chrono::{TimeZone, Utc};
 use std::io;
 use std::time::Duration;
-use crate::ping_clients::ping_client::PingClientWarning;
+use std::sync::Once;
+
+static INIT: Once = Once::new();
+
+pub fn initialize() {
+    INIT.call_once(|| {
+        // initialization code here
+        let _ = env_logger::builder().is_test(true).try_init();
+    });
+}
 
 pub fn generate_ping_result_test_samples() -> Vec<PingResult> {
     vec![
@@ -21,7 +29,6 @@ pub fn generate_ping_result_test_samples() -> Vec<PingResult> {
             None,
             None,
         ),
-
         // Timeout
         PingResult::new(
             &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
@@ -36,7 +43,6 @@ pub fn generate_ping_result_test_samples() -> Vec<PingResult> {
             None,
             None,
         ),
-
         // Reachable but got handshake failure
         PingResult::new(
             &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
@@ -48,13 +54,11 @@ pub fn generate_ping_result_test_samples() -> Vec<PingResult> {
             true,
             Duration::from_millis(20),
             false,
-            Some(PingClientWarning::AppHandshakeFailed(Box::new(io::Error::new(
-                io::ErrorKind::ConnectionAborted,
-                "connect aborted",
-            )))),
+            Some(PingClientWarning::AppHandshakeFailed(Box::new(
+                io::Error::new(io::ErrorKind::ConnectionAborted, "connect aborted"),
+            ))),
             None,
         ),
-
         // Reachable but disconnect connection timed out
         PingResult::new(
             &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
@@ -66,13 +70,11 @@ pub fn generate_ping_result_test_samples() -> Vec<PingResult> {
             true,
             Duration::from_millis(20),
             false,
-            Some(PingClientWarning::DisconnectFailed(Box::new(io::Error::new(
-                io::ErrorKind::TimedOut,
-                "disconnect timeout",
-            )))),
+            Some(PingClientWarning::DisconnectFailed(Box::new(
+                io::Error::new(io::ErrorKind::TimedOut, "disconnect timeout"),
+            ))),
             None,
         ),
-
         // Failed to reach remote
         PingResult::new(
             &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
@@ -85,12 +87,11 @@ pub fn generate_ping_result_test_samples() -> Vec<PingResult> {
             Duration::from_millis(0),
             false,
             None,
-            Some(PingFailed(Box::new(io::Error::new(
+            Some(PingClientError::PingFailed(Box::new(io::Error::new(
                 io::ErrorKind::ConnectionRefused,
                 "connect failed",
             )))),
         ),
-
         // Failed to create local resources for ping, such as cannot bind address
         PingResult::new(
             &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
@@ -103,7 +104,7 @@ pub fn generate_ping_result_test_samples() -> Vec<PingResult> {
             Duration::from_millis(0),
             false,
             None,
-            Some(PreparationFailed(Box::new(io::Error::new(
+            Some(PingClientError::PreparationFailed(Box::new(io::Error::new(
                 io::ErrorKind::AddrInUse,
                 "address in use",
             )))),
